#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import

import sys
import os.path
import argparse

from pidmatsync.sync import (Sync, PersonSync, TeamSync, CachedSync,
                             MembershipSync, ApiSource, MailSync, DeptSync, SyncError)
from pidmatsync.connection import RequestMaker, TokenAuthentication

import logging

logger = logging.getLogger('pidmatsync')


def get_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        'matricules',
        nargs='+',
    )
    parser.add_argument(
        '-t',
        '--token',
        dest='token',
    )
    parser.add_argument(
        '-v',
        '--verbose',
        action='store_true',
        default=False,
    )
    parser.add_argument(
        '--template'
    )
    parser.add_argument(
        '-f',
        '--template-file',
        type=open,
        default=None,
    )
    return parser


class Main(object):
    def __init__(self, parser=None, in_stream=sys.stdin, out_stream=sys.stdout, err_stream=sys.stderr):
        self.parser = parser or get_parser()
        self.in_stream = in_stream
        self.out_stream = out_stream
        self.err_stream = err_stream

    def setup_logging(self, verbose):
        logger = logging.getLogger()
        formatter = logging.Formatter(u'%(levelname)8s %(name)16s: %(message)s')

        sh = logging.StreamHandler(self.err_stream)
        sh.setFormatter(formatter)
        sh.setLevel(logging.DEBUG)

        logger.setLevel(logging.DEBUG if verbose else logging.INFO)
        logger.addHandler(sh)
        logging.getLogger('pidmatsync.connection').setLevel(logging.INFO)
        logging.getLogger('requests.packages.urllib3.connectionpool').setLevel(logging.WARNING)

    def __call__(self, args=None):
        options = self.parser.parse_args(args)
        value = self.run(options)

        if isinstance(value, bool):
            value = int(not value)
        elif value is None:
            value = 0

        sys.exit(value)

    def _get_template(self, options):
        if options.template:
            self.in_stream.encoding

            t = options.template
            if sys.version_info < (3, ):
                t = t.decode(self.in_stream.encoding)

            if not t.endswith(u'\n'):
                t += u'\n'
            return t

        if options.template_file:
            template_name = options.template_file
        else:
            template_name = os.path.join(os.path.dirname(__file__), 'default_template.txt')

        with open(template_name, 'rb') as template_file:
            return template_file.read().decode('utf-8')

    def run(self, options):
        self.setup_logging(options.verbose)
        auth = TokenAuthentication(options.token) if options.token else None
        rmaker = RequestMaker(auth, insecure_ssl=False)
        api_source = ApiSource(rmaker, 'https://pasteurid.pasteur.fr/api/v1/')

        sync = Sync(
            teams=CachedSync(TeamSync(api_source, CachedSync(DeptSync(api_source)))),
            persons=PersonSync(api_source, MailSync(api_source)),
            memberships=MembershipSync(api_source),
        )
        template = self._get_template(options)

        persons = []
        for m in options.matricules:
            try:
                bp = sync(m)
            except SyncError as e:
                self.err_stream.write(u'Cannot fetch "{}": {}\n'.format(m, e))
            else:
                persons.append(bp)

        for value in persons:
            self.out_stream.write(template.format(**value._asdict()))


main = Main()


if __name__ == '__main__':
    main()
