#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import collections
import time

try:
    # Python3
    from urllib.parse import quote
except ImportError:
    from urllib import quote

logger = logging.getLogger()


BoundPerson = collections.namedtuple('BoundPerson', ['team', 'person'])
Person = collections.namedtuple('Person', [
    'matricule', 'first_name', 'last_name', 'account', 'uidnumber', 'email',
])
Team = collections.namedtuple('Team', ['name', 'head', 'department', 'code'])
Head = collections.namedtuple('Head', ['first_name', 'last_name'])


class SyncError(Exception):
    pass


class ApiSource(object):
    def __init__(self, rmaker, url):
        assert url.endswith('/')
        self.rmaker = rmaker
        self.url = url

    def __call__(self, url_segment, *args, **params):
        url = self.url + url_segment.format(*(quote(str(a)) for a in args))
        return self.rmaker('GET', url, params=params or None)


class RateLimitedAPISource(object):
    def __init__(self, api_source, max_req_s=100):
        self.api_source = api_source
        self.last_call = 0
        self.delay = 1. / max_req_s

    def __call__(self, *args, **kw):
        delta = self.delay - time.time() - self.last_call
        if delta > 0:
            # Please wait for regulation
            time.sleep(delta)
        return self.api_source(*args, **kw)


class ManyObjectSync(object):
    def __init__(self, api_source, segment, lookup):
        self.segment = segment
        self.api_source = api_source
        self.lookup = lookup

    def __call__(self, uidnumber):
        for m in self.api_source(self.segment + '/', uidnumber=uidnumber):
            if m[self.lookup]:
                return m
        raise SyncError('Can not find a {} {} for {}'.format(self.lookup, self.segment, uidnumber))


class MailSync(ManyObjectSync):
    def __init__(self, api_source):
        self.api_source = api_source

    def __call__(self, uidnumber):
        backup = None
        for m in self.api_source('mail/', uidnumber=uidnumber):
            if m['published']:
                return m['email']
            if m['backup']:
                backup = m['email']
        if backup is not None:
            return backup
        raise SyncError('Can not find a email principal or backup for {}'.format(uidnumber))


def MembershipSync(api_source):
    return ManyObjectSync(api_source, 'membership', 'principal')


class TeamSync(object):
    def __init__(self, api_source, dept_sync):
        self.api_source = api_source
        self.dept_sync = dept_sync

    def __call__(self, group_id):
        group = self.api_source('group/{}/', group_id)
        team = self.api_source('team/{}/', group['team'])
        person = self.api_source('person/{}/', team['head'])
        dept = self.dept_sync(team['parent'])
        return Team(team['name'], Head(person['first_name'], person['last_name']), dept, team['budget_code'])


class DeptSync(object):
    def __init__(self, api_source):
        self.api_source = api_source

    def __call__(self, budget_code):
        dept = self.api_source('team/', budget_code=budget_code)
        return dept['name']


class PersonSync(object):
    def __init__(self, api_source, mail_sync):
        self.api_source = api_source
        self.mail_sync = mail_sync

    def __call__(self, matricule):
        person = self.api_source('person/', matricule=matricule)
        account = self.api_source('account/{}/', person['account'])
        email = self.mail_sync(account['uidnumber'])
        person = Person(
            matricule,
            person['first_name'],
            person['last_name'],
            account['login'],
            account['uidnumber'],
            email,
        )
        return person


class CachedSync(object):
    def __init__(self, source):
        self._source = source
        self._cache = {}

    def __call__(self, oid):
        if oid in self._cache:
            logger.debug('using cache %s(%s)', self._source, oid)
            return self._cache[oid]

        self._cache[oid] = value = self._source(oid)
        return value


class Sync(object):
    def __init__(self, teams, memberships, persons):
        self.teams = teams
        self.memberships = memberships
        self.persons = persons

    def __call__(self, matricule):
        logger.info('Inspecting %s', matricule)
        person = self.persons(matricule)
        membership = self.memberships(person.uidnumber)
        team = self.teams(membership['group'])
        return BoundPerson(team, person)
